/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
var dataTableObj = new DataTable(); 

//types for table
//SINGLE table can only have one row selected, MULTI table can have multiple rows selected
DataTable.SINGLE = 0;
DataTable.MULTI = 1;
DataTable.ID_DELIMITER = ";";
DataTable.TABLE_HEADER_DIV = "table_header_div";

function DataTable()
{
	var tableID;
	var oTable;
	var tableRows = new Map();
	var handler;
	var tableType = DataTable.MULTI;
	var startRowIdx;
	
	var tdStyle = "scm_DataTable_Column";
	var trStyle = "scm_DataTable_Row";
	var trStyleAlt = "scm_DataTable_RowGray";
	
	//this method must be called before the DataTable instance is ready to use
	//id: the full id of the data table
	//extraHandler: customized event handler
	//type: the type of the table (SINGLE/MULTI)
	//startRow: the index of the body rows. if the table does not include header the startRow is 0. otherwise the startRow should be 1. Default value is 0;
	this.init = function(id, extraHandler, type, startRow)
	{
		tableID = id;
		oTable = $(id);
		handler = extraHandler;
		tableType = type;
		if(oTable)
		{
			var oRows = oTable.getElementsByTagName("tr");
			if(startRow == null)
			{
				startRow = 1;
			}
			startRowIdx = startRow;
			for(var i = startRow; i < oRows.length; i++)
			{
				EventHandling.addEventHandler("click", oRows[i], this.onRowClicked, false);
				tableRows.put(oRows[i], new TableRow(oRows[i]));
			}
			//Initialize tableRow's selection state
			this.initTableRowState();
		}
	}
	
	//adjust the header table width
	this.initHeaderDivPaddingWidth = function(bodyId, divId)
	{
		if(divId == null || divId == "")
		{
			divId = DataTable.TABLE_HEADER_DIV;
		}
		var headerDiv = $(divId);
		var bodyTable = $(bodyId);
		if(bodyTable != null && bodyTable.rows.length > 0)
		{
			bodyTable.style.tableLayout = "fixed";
			var divWidth = headerDiv.clientWidth;
			var bodyTableWidth = bodyTable.offsetWidth;
			var paddingRightWidth = 0;
			if(divWidth > bodyTableWidth)
			{
				//it means the body table has the scroll bar
				paddingRightWidth = divWidth - bodyTableWidth;
			}
			headerDiv.style.paddingRight = String(paddingRightWidth) + "px";
		}
		else if(bodyTable != null && bodyTable.rows.length == 0)
		{
			headerDiv.style.paddingRight = "0px";
			bodyTable.style.tableLayout = "auto";
		}
	}
	
	//remove all the table rows
	//this method is often used on table unbound to backend data
	this.removeAllRows = function()
	{
		var rowCount = oTable.rows.length;
		for(var i = 0; i < rowCount - 1; i++)
		{
			oTable.deleteRow(i);
		}
		tableRows.clear();
	}
	
	this.hasRows = function()
	{
		if(tableRows.getSize() > 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	//remove a table row
	//this method is often used on table unbound to backend data
	this.removeRow = function(oRow)
	{
		var oRows = oTable.rows;
		for(var i = 0; i < oRows.length; i++)
		{
			if(oRow == oRows[i])
			{
				tableRows.remove(oRow);
				oTable.deleteRow(i);
				break;
			}
		}
		refreshRowClass();
		this.initTableRowState();
	}
	
	//insert a table row
	//this method is often used on table unbound to backend data
	//this method should be invoked like this: insertRow(key, cellValue1, cellValue2, ...)
	//key is the unique attribute value of each row that can be used for title attribute etc.
	this.insertRow = function()
	{
		var insertIndex = oTable.rows.length;
		var oRow = oTable.insertRow(insertIndex);
		var cellIndex = 0;
		var oCell = oRow.insertCell(cellIndex);
		var key = arguments[0];
		oCell.className = tdStyle;
		cellIndex++;
		
		//insert a radio/checkbox to the first cell
		var oDiv = document.createElement("div");
		var oSpan = document.createElement("span");
		if(tableType == DataTable.SINGLE)
		{
			var radioHTML = "";
			radioHTML += '<input type="radio"';
			radioHTML += ' title="' + key + '" id="';
			radioHTML += tableID + ":" + "radio" + insertIndex;
			radioHTML += '" name="';
			radioHTML += tableID + ":" + "radio";
			radioHTML += '"/>';
			
			oSpan.innerHTML = radioHTML;
		}
		else //tableType == DataTable.MULTI
		{
			var oCheckbox = document.createElement("input");
			oCheckbox.type = "checkbox";
			oCheckbox.id = tableID + ":" + "box" + insertIndex;
			oCheckbox.title = key;
			oSpan.appendChild(oCheckbox);
		}
		oDiv.appendChild(oSpan);
		oCell.appendChild(oDiv);
		
		var count = 0;
		//insert cell values
		//Position 0 is used for argument key, thus starting from 1
		for(var i = 1; i < arguments.length; i++)
		{
			var oSpan = document.createElement("span");
			oSpan.className = "scm_outputText";
			oSpan.innerHTML = arguments[i];
			oCell = oRow.insertCell(cellIndex);
			cellIndex++;
			oCell.className = tdStyle;
			oCell.appendChild(oSpan);
		}
		
		EventHandling.addEventHandler("click", oRow, this.onRowClicked, false);
		tableRows.put(oRow, new TableRow(oRow));
		
		refreshRowClass();
		this.initTableRowState();
		return oRow;
	}
	
	function refreshRowClass()
	{
		var oRows = oTable.rows;
		for(var i = startRowIdx; i < oRows.length; i++)
		{
			if(i%2 != 0)
			{
				oRows[i].className = trStyle;
				tableRows.get(oRows[i]).setNormalStateClass(trStyle);
			}
			else
			{
				oRows[i].className = trStyleAlt;
				tableRows.get(oRows[i]).setNormalStateClass(trStyleAlt);
			}
		}
	}
	
	//usually called before post, 
	//as the table has displayed and hidden checkboxes/radios, should keep the selection state the same between those two
	this.syncCheckBox = function()
	{
		var rowObjs = tableRows.getValues();
		for(var i = 0; i < rowObjs.length; i++)
		{
			oRow = rowObjs[i].getRowElem();
			var oCheckBoxes = getCheckBoxes(oRow);
			
			if(oCheckBoxes.length == 2)
			{
				oCheckBoxes[1].checked = oCheckBoxes[0].checked;
			}			
		}
	}
	
	//set all the hidden checkboxes/radios to unselected
	this.clearHiddenCheckBox = function()
	{
		var rowObjs = tableRows.getValues();
		for(var i = 0; i < rowObjs.length; i++)
		{
			oRow = rowObjs[i].getRowElem();
			var oCheckBoxes = getCheckBoxes(oRow);
			
			if(oCheckBoxes.length == 2)
			{
				oCheckBoxes[1].checked = false;
			}			
		}
	}
	
	//set a hiddencheck boxes state to checked
	//the checkbox is a child of oRow
	this.selectHiddenCheckBox = function(oRow)
	{
		var oCheckBoxes = getCheckBoxes(oRow);
		if(oCheckBoxes.length == 2)
		{
			oCheckBoxes[1].checked = true;
		}	
	}
	
	//Initialize tableRow's selection state
	this.initTableRowState = function()
	{
		var rowObjs = tableRows.getValues();
		for(var i = 0; i < rowObjs.length; i++)
		{
			oRow = rowObjs[i].getRowElem();
			var oCheckBoxes = getCheckBoxes(oRow);
			
			if(oCheckBoxes[0].checked == true)
			{
				rowObjs[i].setSelected();
			}		
		}
	}
	
	//get the checkboxes/radios (hidden/displayed) in oRow
	function getCheckBoxes(oRow)
	{
		var oInputs = oRow.getElementsByTagName("input");
		var oCheckBoxes = new Array();
		for(var j = 0; j < oInputs.length; j++)
		{
			if(oInputs[j].type == "checkbox" || oInputs[j].type == "radio")
			{
				oCheckBoxes.push(oInputs[j]);
			}
		}
		return oCheckBoxes;
	}
	
	
	//event handler for row click
	this.onRowClicked = function()
	{
		var e = arguments[0];
		var target = EventHandling.getTargetElement(e);
		if(target.type == "checkbox" || target.type == "radio")
		{
			if(tableType == DataTable.SINGLE)
			{
				var rowObjs = tableRows.getValues();
				for(var i = 0; i < rowObjs.length; i++)
				{
					if(true == rowObjs[i].isSelected())
					{
						rowObjs[i].setUnselected();
					}
				}
			}
			var clickedRow = getCurrentElem(e);
			var tableRow = tableRows.get(clickedRow);
			tableRow.click();
			if(handler)
			{
				handler.apply();
			}
		}
	}
	
	
	//return all of the row elements
	this.getRowElems = function()
	{
		var rows = new Array();
		var oRows = tableRows.getValues();
		for(var i = 0; i < oRows.length; i++)
		{
			rows.push(oRows[i].getRowElem());
		}
		return rows;
		
	}
	
	//return the row count.
	this.getRowCount = function()
	{
		var rowElems = this.getRowElems();
		return rowElems.length;
	}

	//return selected row elements
	this.getSelectedRowElems = function()
	{
		var selectedRows = new Array();
		var oRows = tableRows.getValues();
		for(var i = 0; i < oRows.length; i++)
		{
			
			if(true == oRows[i].isSelected())
			{
				selectedRows.push(oRows[i].getRowElem());
			}
		}
		return selectedRows;
	}
	
	//helper function, to get the row being clicked
	function getCurrentElem(e)
	{
		var currentElem;
		currentElem = (e.currentTarget) ? e.currentTarget : getParentNodeByName(e.srcElement,"TR");
		return currentElem;
	}
	
	//true if at least one row is selected
	this.isRowSelected = function()
	{
		var oRow = tableRows.getValues();
		for(var i = 0; i < oRow.length; i++)
		{
			if(true == oRow[i].isSelected())
			{
				return true;
			}
		}
		return false;
	}
	
	//get the count of selected rows
	this.getSelectedRowCount = function()
	{
		var count = 0;
		var rowObjs = tableRows.getValues();
		for(var i = 0; i < rowObjs.length; i++)
		{
			if(true == rowObjs[i].isSelected())
			{
				count++;
			}
		}
		return count;
	}
	
	//set a row as selected by id
	this.setSelectedRow = function(id)
	{
		var rowObjs = tableRows.getValues();
		for(var i = 0; i < rowObjs.length; i++)
		{
			var oRow = rowObjs[i].getRowElem();
			var oInputs = oRow.cells[0].getElementsByTagName("input");
			for(var j = 0; j < oInputs.length; j++)
			{
				if(oInputs[j].type == "hidden")
				{
					if(id == oInputs[j].value)
					{
						var oCheckBoxes = getCheckBoxes(oRow);
						oCheckBoxes[0].checked = true;
						rowObjs[i].setSelected();
					}
				}
			}
		}
	}
	
	//return selected ids 
	this.getSelectedIds = function()
	{
		var ids = "";
		var rowObjs = tableRows.getValues();
		for(var i = 0; i < rowObjs.length; i++)
		{
			if(true == rowObjs[i].isSelected())
			{
				var oRow = rowObjs[i].getRowElem();
				var oInputs = oRow.cells[0].getElementsByTagName("input");
				for(var j = 0; j < oInputs.length; j++)
				{
					if(oInputs[j].type == "hidden")
					{
						if(ids == "")
						{
							ids = oInputs[j].value;
						}
						else
						{
							ids = ids + DataTable.ID_DELIMITER + oInputs[j].value;
						}
					}
				}
			}
		}
		return ids;
	}
	
	//return encoded selected ids.
	this.getEncodedSelectedIds = function()
	{
		var oldIds = this.getSelectedIds();
		var oldArray = oldIds.split(DataTable.ID_DELIMITER);
		var ids = "";
		for(var i=0; i < oldArray.length; i++)
		{
			if(ids=="")
			{
				ids = encodeURIComponent(oldArray[i]);
			}else{
				ids = ids + DataTable.ID_DELIMITER + encodeURIComponent(oldArray[i]);
			}
		}
		return ids;
	}
	
}

//class that stands for a single table row
function TableRow(trObject)
{
	var UNSELECTED = 0;
	var SELECTED = 1;
	
	var status = UNSELECTED;
	
	//the reference for the dom element
	var row = trObject;
	var className = row.className;	
	
	var selectedClass = "scm_DataTableSelected_Row";
	
	this.setNormalStateClass = function(name)
	{
		className = name;
	}
	
	
	//get the row element
	this.getRowElem = function()
	{
		return row;
	}
	
	
	//called by DataTable when a corresponding table row is clicked
	this.click = function()
	{
		if(status == UNSELECTED)
		{
			status = SELECTED;
		}
		else
		{
			status = UNSELECTED;
		}
		setRowClass();
	}
	
	//set row class
	function setRowClass()
	{
		if(status == UNSELECTED)
		{
			row.className = className;
		}
		else
		{
			row.className = selectedClass;
		}
	}
	
	//true if a row is selected
	this.isSelected = function()
	{
		if(status == UNSELECTED)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	//set the row's state to UNSELECTED
	this.setUnselected = function()
	{
		status = UNSELECTED;
		row.className = className;
	}
	
	//set the row's state to SELECTED
	this.setSelected = function()
	{
		status = SELECTED;
		row.className = selectedClass;
	}
}
