/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

// ------------------------------------------ OBJECTS ---------------------------------------------

function DiskObject(id, capacity, imagePath, displayName)
{
    this.id = id;
    this.capacity = capacity;
    this.imagePath = imagePath;
    this.displayName = displayName;
}

// ------------------------------------------ ADDING ---------------------------------------------

function addDisks()
{
	// Get a list of checked ids		
	var arrayOfIds = getCheckedValues("availableDisks");			
	
	for(var x = 0; x < arrayOfIds.length; x++)
	{
		// Only add if we havent added our maximum number
		if(getMaximumNumberOfDisks(selectedRaidLevel) == NO_MAX_LIMIT || getAddedDisks().length < getMaximumNumberOfDisks(selectedRaidLevel))
		{
			addDisk(arrayOfIds[x]);
		}
	}
	
	fixUpRowStyles();
	refreshNumbers();
	updateAvailableDisksSelectedCount();
	updateWizardControls();
	tableLayoutAdjustment();
	if(parent && parent.window.frames["dynamicFrame"])
	{
		parent.resizeFrameHeight("dynamicFrame", "dynamicFrame");
	}
}

function addDisk(id)
{
	// Check if we are adding into an empty table
	if(getAddedDisks().length == 0)
	{
		// Remove the message cell
		var addedDisksTable = document.getElementById("addedDisksTable");
		addedDisksTable.deleteRow(1);
	}
	
	// Get the disk object
	var disk = availableDisksMap.get(id);
	
	// Remove it from the available map and add it to the added map
	availableDisksMap.remove(id);
	addedDisksMap.put(id, disk);
	
	// Remove the row from the available table
	var availableDisksTable = document.getElementById("availableDisksTable");
	var oldRow = document.getElementById(disk.id);
	
	availableDisksTable.deleteRow(getRowIndex(oldRow, availableDisksTable));
	
	// Add a new row to the added disks table
	var addedDisksTable = document.getElementById("addedDisksTable");
	var newRow = addedDisksTable.insertRow(addedDisksTable.rows.length);
	newRow.id = disk.id;
	
	// Make a cell for the checkbox
	var newSelectCell = newRow.insertCell(0);
	newSelectCell.className = "scm_DataTable_Column scm_outputText";
	newSelectCell.innerHTML = "<input name='addedDisks' type='checkbox' title='"+disk.displayName+"' value='" + disk.id + "' onclick='updateAddedDisksSelectedCount()'>";
	newSelectCell.align = "center";
	newSelectCell.width = "15%";
	
	// Make a cell for the capacity
	var newCapacityCell = newRow.insertCell(1);
	newCapacityCell.className = "scm_DataTable_Column scm_outputText";
	newCapacityCell.innerHTML = formatDecimal(disk.capacity);
	newCapacityCell.align = "right";
	newCapacityCell.width = "30%";
	
	// Make a cell for the name
	var newNameCell = newRow.insertCell(2);
	newNameCell.className = "scm_DataTable_Column scm_outputText";
	var image = document.createElement("img");
	image.setAttribute("src", disk.imagePath);
	var name = document.createTextNode(" " + disk.displayName);
	newNameCell.appendChild(image);
	newNameCell.appendChild(name);
	newNameCell.width = "55%";
}

// ------------------------------------------ REMOVING ---------------------------------------------

function removeDisks()
{
	// Get a list of checked ids
	var arrayOfIds = getCheckedValues("addedDisks");
	
	for(var x = 0; x < arrayOfIds.length; x++)
	{	
		removeDisk(arrayOfIds[x]);
	}
	
	fixUpRowStyles();
	refreshNumbers();
	updateAddedDisksSelectedCount();
	updateAvailableDisksSelectedCount();
	updateWizardControls();
	tableLayoutAdjustment();
	if(parent && parent.window.frames["dynamicFrame"])
	{
		parent.resizeFrameHeight("dynamicFrame", "dynamicFrame");
	}
}

function removeDisk(id)
{
	// Get the disk object
	var disk = addedDisksMap.get(id);
	
	// Remove it from the available map and add it to the added map
	addedDisksMap.remove(id);
	availableDisksMap.put(id, disk);
	
	// Remove the row from the added table
	var addedDisksTable = document.getElementById("addedDisksTable");
	var oldRow = document.getElementById(disk.id);
	
	addedDisksTable.deleteRow(getRowIndex(oldRow, addedDisksTable));
	
	// Add a new row to the added disks table
	var availableDisksTable = document.getElementById("availableDisksTable");
	var newRow = availableDisksTable.insertRow(availableDisksTable.rows.length);
	newRow.setAttribute("id", disk.id);
	
	// Make a cell for the checkbox
	var newSelectCell = newRow.insertCell(0);
	newSelectCell.className = "scm_DataTable_Column scm_outputText";
	newSelectCell.innerHTML = "<input name='availableDisks' type='checkbox' title='"+disk.displayName+"' value='" + disk.id + "' onclick='updateAvailableDisksSelectedCount()'>";
	newSelectCell.align = "center";
	newSelectCell.width = "15%";
	
	// Make a cell for the capacity
	var newCapacityCell = newRow.insertCell(1);
	newCapacityCell.className = "scm_DataTable_Column scm_outputText";
	newCapacityCell.innerHTML = formatDecimal(disk.capacity);
	newCapacityCell.align = "right";
	newCapacityCell.width = "30%";
	
	// Make a cell for the name
	var newNameCell = newRow.insertCell(2);
	newNameCell.className = "scm_DataTable_Column scm_outputText";
	var image = document.createElement("img");
	image.setAttribute("src", disk.imagePath);
	var name = document.createTextNode(" " + disk.displayName);
	newNameCell.appendChild(image);
	newNameCell.appendChild(name);
	newNameCell.width = "55%";
	
	// Check if we removed all drives
	if(getAddedDisks().length == 0)
	{
		// Add the empty message
		var addedDisksTable = document.getElementById("addedDisksTable");
		
		var newRow = addedDisksTable.insertRow(addedDisksTable.rows.length);
		
		var newMessageCell = newRow.insertCell(0);
		newMessageCell.setAttribute("colSpan", "3");
		newMessageCell.className = "scm_DataTable_Column scm_outputText";
		newMessageCell.innerHTML = document.getElementById("view:form1:NLS_NoDisksAdded").innerHTML;
	}
}

// ------------------------------------------ UTIL FUNCTIONS ---------------------------------------------

function getRowIndex(row, table)
{
	for(var x = 0; x < table.rows.length; x++)
	{
		if(table.rows[x].id == row.id)
		{
			return x;
		}
	}
	
	return -1;
}

function getAvailableDisks()
{
	return availableDisksMap.getValues();
}

function getAddedDisks()
{
	return addedDisksMap.getValues();
}

function getCheckedValues(checkboxNames)
{
	var checkboxes = document.getElementsByName(checkboxNames);
	
	var arrayOfIds = new Array();
	
	for(var x = 0; x < checkboxes.length; x++)
	{
		if(checkboxes[x].checked)
		{
			arrayOfIds.push(checkboxes[x].value);
		}
	}
	
	return arrayOfIds;
}

// ------------------------------------------ FIXING/REFRESHING ---------------------------------------------

function fixUpRowStyles(table)
{
	var availableDisksTable = document.getElementById("availableDisksTable");
	var addedDisksTable = document.getElementById("addedDisksTable");
	
	var tables = new Array(availableDisksTable, addedDisksTable);
	
	for(var i = 0; i < tables.length; i++)
	{
		var rows = tables[i].rows;
		
		// Start from 1
		for(var x = 1; x < rows.length; x++)
		{
		
			if(x % 2 == 1)
			{
				rows[x].className = "scm_DataTable_Row";
			}
			else
			{
				rows[x].className = "scm_DataTable_RowGray";
			}
		}
	}
}

function refreshNumbers()
{
	var addedDisks = getAddedDisks();
	var isIR = false;
	try{
		if($("view:form1:is_IR").innerHTML == "true")
			isIR = true;
	}catch(e){
	}
	
	// Set the added number of disks
	document.getElementById("view:form1:addedNumberOfDrivesValue").innerHTML = addedDisks.length;
	
	// Set the total pool capacity
	if(addedDisks.length >= getMinimumNumberOfDisks(selectedRaidLevel,isIR))
	{
		// Show both fields
		document.getElementById("view:form1:totalPoolCapacityValue").style.display = "inline";
		document.getElementById("view:form1:totalPoolCapacityValueGB").style.display = "inline";
		
		var minCapacity = Number.MAX_VALUE;
    	for(i in addedDisks)
    	{
    		if(addedDisks[i].capacity < minCapacity)
    		{
    			minCapacity = addedDisks[i].capacity;
    		}
    	}
		document.getElementById("view:form1:totalPoolCapacityValue").innerHTML = formatDecimal(parseFloat(getEstimatedCapacity(selectedRaidLevel, addedDisks.length, minCapacity)).toFixed(3));
	}
	else
	{
		// We cant calculate the capacity because we dont have enough drives
		document.getElementById("view:form1:totalPoolCapacityValue").innerHTML = "";
		
		// Hide both fields
		document.getElementById("view:form1:totalPoolCapacityValue").style.display = "none";
		document.getElementById("view:form1:totalPoolCapacityValueGB").style.display = "none";
	}
	
	// Get the available disks and sum the remaining system capacity
	var availableDisks = getAvailableDisks();
	
	var sum = 0;
	
	for(var x = 0; x < availableDisks.length; x++)
	{
		sum += availableDisks[x].capacity;
	}
	
	document.getElementById("view:form1:remainingSystemCapacityValue").innerHTML = formatDecimal(parseFloat(sum).toFixed(3));
}

// ------------------------------------------ SELECTED COUNTS ---------------------------------------------
function updateAvailableDisksSelectedCount()
{
	var numberSelected = getCheckedValues("availableDisks").length;
	
	document.getElementById("view:form1:availableTableSelectedValue").innerHTML = numberSelected;
	
	// If there are none selected, disable the add control
	// Also, if we have added our maximum number, then disable the add control
	if(numberSelected == 0 || getAddedDisks().length == getMaximumNumberOfDisks(selectedRaidLevel))
	{
		document.getElementById("addControl").disabled = true;
	}
	else
	{
		document.getElementById("addControl").disabled = false;
	}
}

function updateAddedDisksSelectedCount()
{	
	var numberSelected = getCheckedValues("addedDisks").length;
	
	document.getElementById("view:form1:addedTableSelectedValue").innerHTML = numberSelected;
	
	// If there are none selected, disable the add control
	if(numberSelected == 0)
	{
		document.getElementById("removeControl").disabled = true;
	}
	else
	{
		document.getElementById("removeControl").disabled = false;
	}
}

// ------------------------------------------ VIEW SUMMARY FUNCTIONS ---------------------------------------------

function showWaitMessage(waitMessage)
{
	$("mainContent").style.visibility = "hidden";

	var floatMessage = new FloatMessage(FloatMessage.CREATE_VOLUME_CONTEXT, FloatMessage.STATIC, waitMessage);
	floatMessage.start();
	
	return true;
}

function launchConfirmationModal(thisEvent)
{
	// Make input fields for each of the added disks so that they may be passed to the server
	var inputFields = "";
	
	var diskIds = getAddedDisks();
	
	for(var x = 0; x < diskIds.length; x++)
	{
		inputFields += "<input type='hidden' name='diskIds' value='" + diskIds[x].id + "'>";
	}
	
	document.getElementById("selectedIdsDiv").innerHTML = inputFields;
	
	/**Defect 14677: Pool name input is not shown per UCD SCM_MR_GUI_HLD_10_02_08. Code logic is ready for all actions, to restore the function, simply uncomment the content below
	var poolName = $("view:form1:poolname_input").value;
	
	if(validatePoolName(poolName))
	{
		displayValidationMsg(false);
	**/
		//submit current page and open the confirmation dialog.
		panelDialogObj.launchDialog("view:form1");
		var title = $("view:confirmation_dialog_title").innerHTML;
		//set the dialog title
		setConfirmationDialogTitle(title);
	/**Defect 14677: Pool name input is not shown per UCD SCM_MR_GUI_HLD_10_02_08. Code logic is ready for all actions, to restore the function, simply uncomment the content below
	}
	else
	{
		displayValidationMsg(true);
		
		EventHandling.preventReturnValue(thisEvent);
		// If this page is part of a dymanic frame in a shell, resize the frames
		if(parent && parent.window.frames["dynamicFrame"])
		{
			parent.resizeFrameHeight("dynamicFrame", "dynamicFrame");
		}
	}
	**/
}

//set the panel dialog title
function setConfirmationDialogTitle(title)
{
	panelDialogObj.setPanelDialogTitle("view", title);
};
//validate whether the pool name is valid.
function validatePoolName(poolName)
{
	var regex = /\w+/;  //equals to /[A-Za-z0-9_]+/
	
	if(poolName == "")
	{
		$("view:form1:poolname_errormsg").innerHTML = $("view:form1:null_poolname_msg").innerHTML;
		return false;
	}
	else
	{
		if(poolName != regex.exec(poolName))
		{
			$("view:form1:poolname_errormsg").innerHTML = $("view:form1:incorrect_poolname_msg").innerHTML;
			return false;
		}
		else
		{
			//check whether the pool name has already existed. 
			var allExistingNames = $("view:form1:all_existing_poolnames").innerHTML;
			
			if("" == allExistingNames)
			{
				return true;
			}
			else
			{
				var poolNames =  allExistingNames.split(",");
				for(i in poolNames)
				{
					if(poolName.toUpperCase() == poolNames[i].toUpperCase())
					{
						$("view:form1:poolname_errormsg").innerHTML = $("view:form1:duplicate_poolname_msg").innerHTML;
						return false;
					}
				}
				return true;
			}
		}
	}
}

function displayValidationMsg(flag)
{
	if(flag)
	{
		$("poolname_errormsg_tr").style.display = "";
	}else{
		$("poolname_errormsg_tr").style.display = "none";
	}
}

// ------------------------------------------ WIZARD CONTROLS ---------------------------------------------
function updateWizardControls()
{
	var isIR = false;
	try{
		if($("view:form1:is_IR").innerHTML == "true")
			isIR = true;
	}catch(e){
	}
	if(getAddedDisks().length >= getMinimumNumberOfDisks(selectedRaidLevel, isIR))
	{
		// Enable next if we have enough disks for the pool
		$("view:form1:btn_next").disabled = false;
	}
	else
	{
		// Disable next if we dont have enough disks for the pool
		$("view:form1:btn_next").disabled = true;
	}
}

function disableNextBtn()
{
	$("view:form1:btn_next").disabled = true;
}

function resizeDivHeight(divName, contentId, fixedHeight)
{
    var table = $(contentId);
	var div = $(divName);
    //Problem: In the context of table-layout property of table's style is 'fixed' in IE and all rows of the table are deleted, the table's real height will not be set to zero but still the original value
	//In order to avoid occuring of this problem, table-layout property will be set to 'auto' when there is no row.
	if(table.rows.length == 0)
	{
		table.style.tableLayout = "auto";
	}
    else
    {
		table.style.tableLayout = "fixed";
    }
	var tableHeight = table.scrollHeight;
	var divHeight = div.clientHeight;
	if(tableHeight<fixedHeight)
	{
		div.style.height = "auto";
	}
	else
	{
		div.style.height = fixedHeight+"px";			
	}
}

function tableLayoutAdjustment()
{
    resizeDivHeight("availableDisks_table_content_panel","availableDisksTable","200");
	resizeDivHeight("addedDisks_table_content_panel","addedDisksTable","200");
	dataTableObj.initHeaderDivPaddingWidth('availableDisksTable','availableDisks_table_header_div');
	dataTableObj.initHeaderDivPaddingWidth('addedDisksTable','addedDisks_table_header_div');
}
