/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

// Constant to indicate 
var NO_MAX_LIMIT = Number.MAX_VALUE;

function getEstimatedCapacity(raidLevel, numberOfDisks, capacityOfEachDisk)
{
	// Make sure that we have the correct number of disks for this RAID type
	if(numberOfDisks < getMinimumNumberOfDisks(raidLevel) )
	{
		return 0;
	}

	if (raidLevel == RAID0)		// RAID 0 stripes data over disks so the pool capacity the full capacity of the disks
		return numberOfDisks * capacityOfEachDisk;
	else if (raidLevel == RAID1)		// RAID 1 mirrors data between data over disks so the pool capacity is just the capacity of one disk
		return capacityOfEachDisk;
	else if (raidLevel == RAID5) 	// RAID 5 uses a rotaing parity block, so one disk is always used for parity. The pool capacity becomes the capacity of (numberOfDisks - 1) disks
		return (numberOfDisks - 1) * capacityOfEachDisk;
	else if (raidLevel == RAID6)		// RAID 6 uses 2 rotaing parity blocks, so two disks are always used for parity. The pool capacity becomes the capacity of (numberOfDisks - 2) disks
		return (numberOfDisks - 2) * capacityOfEachDisk;
	else  // Error case: unknown raidLevel
		return 0;
}

function getMinimumNumberOfDisks(raidLevel, isIR)
{	
	if( isIR && raidLevel == RAID0){
		return 2; //IR device requires 2 disks for RAID0
	}else if (raidLevel == RAID0) 	// Normally 2, but we have a requirement to allow one disk pools such that single disks can be seen by the OS.
		return 1;
	else if (raidLevel == RAID1)
		return 2;
	else if (raidLevel == RAID5)
	 	return 3;
	else if (raidLevel == RAID6)
		return 3;
	else return 100;								

}


function getMaximumNumberOfDisks(raidLevel)
{	
	if (raidLevel == RAID0) 	// Normally 2, but we have a requirement to allow one disk pools such that single disks can be seen by the OS.
		return NO_MAX_LIMIT;
	else if (raidLevel == RAID1)
		return 2;
	else if (raidLevel == RAID5)
	 	return NO_MAX_LIMIT;
	else if (raidLevel == RAID6)
		return NO_MAX_LIMIT;
	else return NO_MAX_LIMIT;								

}
