/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

var dvObj = new DefineVolumes();
var VOLUME_NAME_MAX_LENGTH = 12;

function DefineVolumes()
{
	var oStripeSelect;
	var oCapacity;
	var oQuantity;
	var oName;
	var oNameSuffix;
	var oVolumeTable;
	var oVolumeJson;
	var volumeTableObj; 
	var totalPoolCapacity;
	var availablePoolCapacity;
	
	var btn_next;
	
	var validator = new DefineVolumesClientValidator(this);
	
	var volumeMap = new Map();
	
	this.init = function()
	{
		oStripeSelect = $("view:form1:stripe_menu");
		oCapacity = $("capacity_input");
		oQuantity = $("quantity_input");
		oName = $("name_input");
		oNameSuffix = $("namesuffix_input");
		oVolumeTable = $("view:form1:volumes_table");
		oVolumeJson = $("view:form1:volume_json");
		oIsIR = false;
		try{
			if($("view:form1:is_IR").innerHTML=="true")
				oIsIR = true;
		}catch(e){}
		
		
		totalPoolCapacity = (parseFloat($("view:form1:available_capacity_value").innerHTML)/(1024*1024*1024)).toFixed(3);
		availablePoolCapacity = totalPoolCapacity;
		volumeTableObj = new DataTable(); 
		volumeTableObj.init("view:form1:volumes_table", this.updateRemoveButton, DataTable.MULTI);
		
		$("view:form1:btn_remove").disabled = true;
		btn_next = $("view:form1:btn_next");
		btn_next.disabled = "true";
		
		if(oIsIR){
			oName.disabled = true;
			oName.value = "StorageVolume_";
			oQuantity.focus();
			//oQuantity.disabled = true;
			//oQuantity.value = "1";
		}else{
			oName.focus();
			oName.value = "";
			oName.disabled = false;
			//oCapacity.focus();
		}
		
		document.body.scrollTop = 0;
		
		var restored = restorePrevDefinition();
		
		//if no data is restored, update percentage graph only
		updateCapacityDisplay(!restored);
		this.updateNextButton();
	}
	
	function restorePrevDefinition()
	{
		var definitionStr = oVolumeJson.value;
		if(definitionStr && "" != definitionStr)
		{
			var jsonObjs = JSON.parse(definitionStr);
			if(jsonObjs)
			{
				for(i in jsonObjs)
				{
					var capacity = parseFloat(jsonObjs[i].capacity);
					var stripeSize = parseInt(jsonObjs[i].stripeSize);
					var name = jsonObjs[i].name;
					addVolume(capacity, stripeSize, name);
				}
			}
			return true;
		}
		return false;
	}
	
	function addVolume(capacity, stripeSize, name)
	{
		var volume = {};
		volume.stripeSize = stripeSize;
		volume.name = name;
		volume.capacity = capacity;
		if(volumeMap.get(volume.name))
		{
			return;
		}
		volumeMap.put(volume.name, volume);
		var oRow = volumeTableObj.insertRow(volume.name,formatDecimal(volume.capacity), volume.name);
		oRow.getElementsByTagName('td')[0].align = "left";
		oRow.getElementsByTagName('td')[0].style.padding = "0px";
		oRow.getElementsByTagName('td')[1].align = "right";
		oRow.getElementsByTagName('td')[2].align = "left";		
		oRow.getElementsByTagName('td')[0].width = "8%";
		oRow.getElementsByTagName('td')[1].width = "35%";
		oRow.getElementsByTagName('td')[2].width = "57%";
		//adjust the header table width
		dataTableObj.initHeaderDivPaddingWidth("view:form1:volumes_table");
	}
	
	this.addVolumes = function()
	{
		if(!validator.validate())
		{
			return;
		}
		
		var capacity = parseDecimal(oCapacity.value);
		var quantity = parseDecimal(oQuantity.value);
		var stripeSize = parseInt(oStripeSelect.value);
		var name = oName.value;
		var nameSuffix = parseInt(oNameSuffix.value);
		
		if(quantity > 1)
		{
			for(var i = 0; i < quantity; i++)
			{
				addVolume(capacity, stripeSize, name + (i + nameSuffix));
			}
		}
		if(quantity == 1)
		{
			addVolume(capacity, stripeSize, name);
		}
		updateCapacityDisplay(false);
		this.updateNextButton();
	}
	
	this.updateRemoveButton = function()
	{
		var oRows = volumeTableObj.getSelectedRowElems();
		if(!oRows || 0 == oRows.length)
		{
			$("view:form1:btn_remove").disabled = true;
		}
		else
		{
			$("view:form1:btn_remove").disabled = false;
		}
	}
	
	this.updateNextButton = function()
	{
		if(volumeTableObj.hasRows())
		{
			btn_next.disabled = false;
		}
		else
		{
			btn_next.disabled = true;
		}
	}
	
	this.removeVolumes = function()
	{
		var oRows = volumeTableObj.getSelectedRowElems();
		var len = oRows.length;
		if(oRows)
		{
			for(var i = 0; i < len; i++)
			{
				var volumeName = getDeepestChild(oRows[i].cells[2]).innerHTML;
				volumeMap.remove(volumeName);
				volumeTableObj.removeRow(oRows[i]);
			}
		}
		updateCapacityDisplay(false);
		this.updateRemoveButton();
		this.updateNextButton();
		//adjust the header table width
		dataTableObj.initHeaderDivPaddingWidth("view:form1:volumes_table");
	}
	
	this.createVolumes = function()
	{
		var volumeStr = JSON.stringify(volumeMap.getValues());
		oVolumeJson.value = volumeStr;
	}
	
	this.onQuantityChange = function(quantity)
	{
		if(!isNaN(quantity) && parseInt(quantity) > 1)
		{
			oNameSuffix.disabled = false;
		}
		else
		{
			oNameSuffix.disabled = true;
		}
	}
	
	function updateCapacityDisplay(updatePercentageOnly)
	{
		var usedPercentage;
		var leftPercentage;
		var usedCapacity = 0;
		var leftCapacity = 0;
		if(0 == totalPoolCapacity)
		{
			leftPercentage = 0;
		}
		else
		{
			var volumes = volumeMap.getValues();
			for(i in volumes)
			{
				usedCapacity += volumes[i].capacity;
			}
			usedCapacity = parseFloat(usedCapacity.toFixed(3));
			leftCapacity = parseFloat((availablePoolCapacity - usedCapacity).toFixed(3));
			leftPercentage = parseInt(leftCapacity / totalPoolCapacity * 100);
		}
		usedPercentage = 100 - leftPercentage;
		//need to hide 0% width td, otherwise it still has a bit width
		if(0 == usedPercentage)
		{
			$("used_cap_td").style.display = "none";
		}
		else
		{
			$("used_cap_td").style.display = "";
		}
		$("used_cap_td").style.width = usedPercentage + "%";
		if(0 == leftPercentage)
		{
			$("available_cap_td").style.display = "none";
		}
		else
		{
			$("available_cap_td").style.display = "";
		}
		$("available_cap_td").style.width = leftPercentage + "%";
		
		if(!updatePercentageOnly)
		{
			$("view:form1:available_capacity").innerHTML = formatDecimal(leftCapacity);
			$("used_cap_value").innerHTML = formatDecimal(usedCapacity);
		}
	}
	
	this.getLeftCapacity = function()
	{
		var volumes = volumeMap.getValues();
		var usedCapacity = 0.0;
		for(i in volumes)
		{
			usedCapacity += volumes[i].capacity;
		}
		useCapacity = parseFloat(usedCapacity.toFixed(3));
		var leftCapacity = parseFloat((availablePoolCapacity - usedCapacity).toFixed(3));
		return leftCapacity;
	}
	
	this.getRightTableRowCount = function()
	{
		return volumeTableObj.getRowCount();
	}
	
	this.updateVolumeNameSample = function()
	{
		var name = oName.value;
		var quantity = parseInt(oQuantity.value);
		var suffix = parseInt(oNameSuffix.value);
		var sample = "";
		if(!isNaN(quantity))
		{
			if(quantity > 1 && !isNaN(suffix))
			{
				sample = name + suffix + " - " + name + (suffix + quantity - 1);
			}
			else if(quantity == 1)
			{
				sample = name;
			}
		}
		
		$("name_sample_value").innerHTML = sample;
	}
}

function DefineVolumesClientValidator(pageLogic)
{
	var ruleMap = new Map();
	var pageObj = pageLogic;
	
	var capacityRule = {};
	capacityRule.isNumber = true;
	capacityRule.allowEmpty = false;
	//allow decimal number only
	capacityRule.format = "^([1-9]{1}\\\d*|0)(\\\.\\\d{0,3})?$";
	capacityRule.max = 2048.0;
	capacityRule.special = capacitySpecialValidation;
	ruleMap.put("capacity_input", capacityRule);
	
	var quantityRule = {};
	quantityRule.isNumber = true;
	quantityRule.allowEmpty = false;
	quantityRule.format = "^[1-9]{1}\\\d*$";
	quantityRule.max = 16;
	quantityRule.special = quantitySpecialValidation;
	ruleMap.put("quantity_input", quantityRule);
	
	var nameRule = {};
	nameRule.allowEmpty = false;
	nameRule.format = "^[_0-9A-Za-z]+$";
	nameRule.special = nameSpecialValidation;
	ruleMap.put("name_input", nameRule);
	
	var suffixRule = {};
	suffixRule.allowEmpty = false;
	suffixRule.format = "^(0|([1-9]{1}\\\d*))$";
	ruleMap.put("namesuffix_input", suffixRule);
	
	function capacitySpecialValidation()
	{
		var capacity = parseDecimal($("capacity_input").value);
		var quantity = parseDecimal($("quantity_input").value);
		var total = parseFloat(capacity * quantity).toFixed(3);
		if(pageObj.getLeftCapacity() < total)
		{
			var msg = $("view:form1:text_total_capacity_exceed").innerHTML;
			showErrorMessage("capacity_input_err_special", msg);
			return false;
		}
	}
	
	function quantitySpecialValidation()
	{
		
		var leftQuantity = parseDecimal($("quantity_input").value);
		var rightQuantity = Number(pageObj.getRightTableRowCount());
		var allQuantity = leftQuantity + rightQuantity;
		var remainQuantity = Number($("view:form1:remain_volumes_count").innerHTML);
		if(allQuantity > remainQuantity)
		{
			showErrorMessage("quantity_input_err_max");
			return false;
		}
	}
	
	function nameSpecialValidation()
	{
		var quantity = parseDecimal($("quantity_input").value);
		var suffix = parseInt($("namesuffix_input").value);
		var name = $("name_input").value;
		var maxNameStr = name + (suffix + quantity - 1);
		if(maxNameStr.length > VOLUME_NAME_MAX_LENGTH)
		{
			showErrorMessage("name_input_err_length");
			return false;
		}
	}
	
	this.validate = function()
	{
		clear();
		var elementIds = ruleMap.getKeys();
		var passValidation = true;
		for(i in elementIds)
		{
			if($(elementIds[i]).disabled)
			{
				//skip validating this input
				continue;
			}
			var rule = ruleMap.get(elementIds[i]);
			var valueToValidate = $(elementIds[i]).value;
			
			if(false == rule.allowEmpty)
			{
				if("" == valueToValidate)
				{
					passValidation = false;
					$(elementIds[i]).setAttribute("aria-invalid","true");
					showErrorMessage(elementIds[i] + "_err_empty");
					continue;
				}
			}
			
			if(rule.format)
			{
				//if an input has max or min limitation, its value is a number
				var value = valueToValidate;
				if(rule.isNumber && rule.isNumber == true)
				{
					var value = parseDecimal(value);
					if(value == null)
					{
						passValidation = false;
						$(elementIds[i]).setAttribute("aria-invalid","true");
						showErrorMessage(elementIds[i] + "_err_format");
						continue;
					}
				}
				value = new String(value);
				if(null == value.match(rule.format))
				{
					passValidation = false;
					$(elementIds[i]).setAttribute("aria-invalid","true");
					showErrorMessage(elementIds[i] + "_err_format");
					continue;
				}
			}
			
			if(rule.max)
			{
				if(null == parseDecimal(valueToValidate) || parseDecimal(valueToValidate) > rule.max)
				{
					passValidation = false;
					$(elementIds[i]).setAttribute("aria-invalid","true");
					showErrorMessage(elementIds[i] + "_err_max");
					continue;
				}
			}
			
			if(rule.min)
			{
				if(null == parseDecimal(valueToValidate) || parseDecimal(valueToValidate) < rule.min)
				{
					passValidation = false;
					$(elementIds[i]).setAttribute("aria-invalid","true");
					showErrorMessage(elementIds[i] + "_err_min");
					continue;
				}
			}
		}
		
		if(passValidation)
		{
			for(i in elementIds)
			{
				if($(elementIds[i]).disabled)
				{
					//skip validating this input
					continue;
				}
				var rule = ruleMap.get(elementIds[i]);
			
				if(rule.special)
				{
					var ret = rule.special.call(this);
					if(false == ret)
					{
						passValidation = false;
						$(elementIds[i]).setAttribute("aria-invalid","true");
					}
				}
			}
		}
		
		return passValidation;
	}
	
	function showErrorMessage(messageId, message)
	{
		$(messageId).style.display = "block";
		$(messageId).setAttribute("role", "alert");
		if(message)
		{
			$(messageId).innerHTML = message;
		}
	}
	
	function clearErrorMessages()
	{
		var error_elem_suffix = ["_err_empty", "_err_length", "_err_format", "_err_max", "_err_min", "_err_special"];
		var elementIds = ruleMap.getKeys();
		for(i in elementIds)
		{
			for(j in error_elem_suffix)
			{
				if($(elementIds[i] + error_elem_suffix[j]))
				{
					$(elementIds[i] + error_elem_suffix[j]).style.display = "none";
				}
			}
		}
	}
	
	function clear()
	{
		clearErrorMessages();
		var elementIds = ruleMap.getKeys();
		for(i in elementIds)
		{
			$(elementIds[i]).setAttribute("aria-invalid","false");
		}
	}
}
